﻿using System;
using System.Xml;
using System.Collections.Generic;

namespace Esempio4
{
    class Program
    {
        static void Main(string[] args)
        {
            //Step0();
            //Step1();
            //Step2();
            //Step3();
            //Step4();
            //Step5();
            //Step6();
            Step7();
        }

        private static void Step0()
        {
            XmlTextReader reader = new XmlTextReader("../../Dati0.xml");
            reader.WhitespaceHandling = WhitespaceHandling.None;
            reader.MoveToContent();  //  Salta commenti e dichiarazioni
            reader.ReadStartElement("Gruppo");
            while (reader.IsStartElement("Item"))
            {
                reader.Read();  //  Cosa succede con Read()?
            }
            reader.ReadEndElement();
            reader.Close();
        }

        #region Step1 - Lettura di un documento XML da file mediante XmlReader

        private static void Step1()
        {
            XmlTextReader reader = new XmlTextReader("../../Dati1.xml");
            reader.WhitespaceHandling = WhitespaceHandling.None;
            reader.MoveToContent();  //  Salta commenti e dichiarazioni
            reader.ReadStartElement("Gruppo");
            while (reader.IsStartElement())
            {
                string name = reader.Name;
                //string text = reader.ReadString();
                string text = reader.ReadElementString();
                Console.WriteLine("{0} = \"{1}\"", name, text);
                //reader.Skip();
            }
            reader.ReadEndElement();
            reader.Close();
        }

        #endregion

        #region Step2 - Lettura di un documento XML da file mediante XmlReader

        private static void Step2()
        {
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.IgnoreWhitespace = true;
            settings.ProhibitDtd = false;
            XmlReader reader = XmlReader.Create("../../Dati2.xml", settings);

            //XmlTextReader reader = new XmlTextReader("../../Dati2.xml");
            //reader.WhitespaceHandling = WhitespaceHandling.None;  //  2
            //reader.EntityHandling = EntityHandling.ExpandEntities;  // 3

            while (reader.Read())
            {
                Console.WriteLine("NodeType={0}, name=\"{1}\", value=\"{2}\"",
                    reader.NodeType, reader.Name, reader.Value);  //  1
                if (reader.HasAttributes)
                {
                    //for (int k = 0; k < reader.AttributeCount; k++)  //  4
                    //    Console.WriteLine("  Attribute index={0}, value=\"{1}\"", k, reader.GetAttribute(k));
                    while (reader.MoveToNextAttribute())  //  5
                        Console.WriteLine("  Attribute name=\"{0}\", value=\"{1}\"", reader.Name, reader.Value);
                    reader.MoveToElement();
                }
            }
            reader.Close();
        }

        #endregion

        #region Step3 - Creazione di un nuovo documento XML mediante XmlWriter

        private static void Step3()
        {
            XmlTextWriter writer = new XmlTextWriter("../../Dati3.xml", System.Text.Encoding.UTF8);
            writer.Formatting = Formatting.Indented;
            writer.Indentation = 3;
            writer.WriteStartDocument();
            writer.WriteStartElement("XmlNodeType");
            foreach (string xmlNodeTypeName in Enum.GetNames(typeof(XmlNodeType)))
            {
                //  Valore numerico dell'enumerativo
                int xmlNodeTypeValue = (int) Enum.Parse(typeof(XmlNodeType), xmlNodeTypeName);
                writer.WriteStartElement(xmlNodeTypeName);
                //  Valore come attributo
                writer.WriteAttributeString("value", XmlConvert.ToString(xmlNodeTypeValue));
                //  Valore inserito nell'elemento
                writer.WriteString(XmlConvert.ToString(xmlNodeTypeValue));
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteEndDocument();
            writer.Close();
        }

        #endregion

        #region Step4 - Scansione di tutti i nodi di un documento XML mediante DOM

        private static void Step4()
        {
            XmlDocument doc = new XmlDocument();
            doc.Load("../../Dati2.xml");

            #region
            //XmlTextReader reader = new XmlTextReader("../../Dati2.xml");
            //reader.EntityHandling = EntityHandling.ExpandEntities;
            //doc.Load(reader); 
            #endregion

            WriteToConsole(doc);

            DumpNode(doc, 0);
        }

        #endregion

        #region Step5 - Caricamento di un documento XML da file mediante DOM

        private static void Step5()
        {
            XmlDocument doc = new XmlDocument();
            doc.Load("../../Dati1.xml");
            WriteToConsole(doc);

            Console.WriteLine("doc.OuterXml = \"{0}\"", doc.OuterXml);
            Console.WriteLine("");
            Console.WriteLine("doc.InnerXml = \"{0}\"", doc.InnerXml);
            Console.WriteLine("");
            Console.WriteLine("doc.InnerText = \"{0}\"", doc.InnerText);
            Console.WriteLine("");
            Console.WriteLine("");
            Console.ReadLine();

            //doc.DocumentElement.InnerXml = "<Item id=\"12\">Pluto</Item>";
            //doc.DocumentElement.InnerText = "<Item id=\"12\">Pluto</Item>";

            Console.WriteLine("doc.DocumentElement.OuterXml = \"{0}\"", doc.DocumentElement.OuterXml);
            Console.WriteLine("");
            Console.WriteLine("doc.DocumentElement.InnerXml = \"{0}\"", doc.DocumentElement.InnerXml);
            Console.WriteLine("");
            Console.WriteLine("doc.DocumentElement.InnerText = \"{0}\"", doc.DocumentElement.InnerText);
            Console.WriteLine("");
            Console.WriteLine("");
            Console.ReadLine();

            foreach (XmlNode xmlNode in doc.DocumentElement.ChildNodes)
            {
                Console.WriteLine("xmlNode.OuterXml = \"{0}\"", xmlNode.OuterXml);
                Console.WriteLine("");
                Console.WriteLine("xmlNode.InnerXml = \"{0}\"", xmlNode.InnerXml);
                Console.WriteLine("");
                Console.WriteLine("xmlNode.InnerText = \"{0}\"", xmlNode.InnerText);
                Console.WriteLine("");
            }
        }

        #endregion

        #region Step6 - Creazione di un nuovo documento XML mediante DOM

        private static void Step6()
        {
            XmlDocument doc = new XmlDocument();
            XmlNode node = doc.CreateXmlDeclaration("1.0", "", "");
            doc.AppendChild(node);
            XmlElement root = doc.CreateElement("XmlNodeType");
            doc.AppendChild(root);
            foreach (string xmlNodeTypeName in Enum.GetNames(typeof(XmlNodeType)))
            {
                //  Valore numerico dell'enumerativo
                int xmlNodeTypeValue = (int) Enum.Parse(typeof(XmlNodeType), xmlNodeTypeName);
                XmlElement element = doc.CreateElement(xmlNodeTypeName);
                //  Valore inserito nell'elemento
                element.InnerText = XmlConvert.ToString(xmlNodeTypeValue);
                //  Valore come attributo
                element.SetAttribute("value", XmlConvert.ToString(xmlNodeTypeValue));
                root.AppendChild(element);
            }
            doc.Save("../../Dati3.xml");
            WriteToConsole(doc);
        }

        #endregion

        #region Step7 - Selezione di nodi da un documento XML mediante DOM

        private static void Step7()
        {
            XmlDocument doc = new XmlDocument();
            doc.Load("../../Dati4.xml");
            WriteToConsole(doc);

            Console.WriteLine("Selezione: /Gruppo/Item");
            foreach (XmlNode xmlNode in doc.SelectNodes("/Gruppo/Item"))
            {
                Console.WriteLine(xmlNode.OuterXml);
            }
            Console.WriteLine("");

            Console.WriteLine("Selezione a partire da DocumentElement");
            //foreach (XmlNode xmlNode in doc.DocumentElement.SelectNodes("Item"))
            foreach (XmlNode xmlNode in doc.DocumentElement.SelectNodes("/Gruppo//Item"))
            {
                Console.WriteLine(xmlNode.OuterXml);
            }
            Console.WriteLine("");

            Console.WriteLine("Selezione: /Gruppo/Item[@id >= 5]");
            foreach (XmlNode xmlNode in doc.SelectNodes("/Gruppo/Item[@id >= 5]"))
            {
                Console.WriteLine(xmlNode.OuterXml);
            }
            Console.WriteLine("");

            Console.WriteLine("Selezione: /Gruppo/Item[text() = 'Topolino']");
            foreach (XmlNode xmlNode in doc.SelectNodes("/Gruppo//Item[text() = 'Topolino']"))
            //foreach (XmlNode xmlNode in doc.SelectNodes("/Gruppo/Item[text() = 'Topolino']"))
            {
                Console.WriteLine(xmlNode.OuterXml);
            }
            Console.WriteLine("");
        }

        #endregion

        private static void WriteToConsole(XmlDocument doc)
        {
            doc.Save(Console.Out);
            Console.WriteLine("");
            Console.ReadLine();
        }

        private static void DumpNode(XmlNode xmlNode, int depth)
        {
            Console.Write(new String(' ', depth));
            Console.WriteLine("NodeType={0}, name=\"{1}\", value=\"{2}\"",
                xmlNode.NodeType, xmlNode.Name, xmlNode.Value);
            if (xmlNode.Attributes != null)
            {
                foreach (XmlAttribute xmlAttribute in xmlNode.Attributes)
                {
                    Console.Write(new String(' ', depth));
                    Console.WriteLine("+ Attribute name=\"{0}\", value=\"{1}\"", xmlAttribute.Name, xmlAttribute.Value);
                }
            }

            foreach (XmlNode xmlChildNode in xmlNode.ChildNodes)
            {
                DumpNode(xmlChildNode, depth + 1);
            }
        }
    }
}
